@@ -1,22 +1,26 @@
 #include <filezilla.h>
 
-CConnectCommand::CConnectCommand(CServer const& server, bool retry_connecting /*=true*/)
-	: m_Server(server)
-	, m_retry_connecting(retry_connecting)
+CConnectCommand::CConnectCommand(CServer const& server, Credentials const& credentials, bool retry_connecting)
+	: server_(server)
+	, credentials_(credentials)
+	, retry_connecting_(retry_connecting)
 {
 }
 
-CServer const& CConnectCommand::GetServer() const
+bool CConnectCommand::valid() const
 {
-	return m_Server;
+	if (!server_) {
+		return false;
+	}
+	return true;
 }
 
-CListCommand::CListCommand(int flags /*=0*/)
+CListCommand::CListCommand(int flags)
 	: m_flags(flags)
 {
 }
 
-CListCommand::CListCommand(CServerPath path, wxString subDir, int flags)
+CListCommand::CListCommand(CServerPath path, std::wstring const& subDir, int flags)
 	: m_path(path), m_subDir(subDir), m_flags(flags)
 {
 }
@@ -26,7 +30,7 @@ CServerPath CListCommand::GetPath() const
 	return m_path;
 }
 
-wxString CListCommand::GetSubDir() const
+std::wstring CListCommand::GetSubDir() const
 {
 	return m_subDir;
 }
@@ -76,12 +80,12 @@ bool CFileTransferCommand::Download() const
 	return m_download;
 }
 
-CRawCommand::CRawCommand(const wxString &command)
+CRawCommand::CRawCommand(std::wstring const& command)
 {
 	m_command = command;
 }
 
-wxString CRawCommand::GetCommand() const
+std::wstring CRawCommand::GetCommand() const
 {
 	return m_command;
 }
@@ -91,7 +95,7 @@ CDeleteCommand::CDeleteCommand(const CServerPath& path, std::deque<std::wstring>
 {
 }
 
-CRemoveDirCommand::CRemoveDirCommand(const CServerPath& path, const wxString& subDir)
+CRemoveDirCommand::CRemoveDirCommand(const CServerPath& path, std::wstring const& subDir)
 	: m_path(path), m_subDir(subDir)
 {
 }
@@ -111,8 +115,8 @@ bool CMkdirCommand::valid() const
 	return !GetPath().empty() && GetPath().HasParent();
 }
 
-CRenameCommand::CRenameCommand(const CServerPath& fromPath, const wxString& fromFile,
-							   const CServerPath& toPath, const wxString& toFile)
+CRenameCommand::CRenameCommand(CServerPath const& fromPath, std::wstring const& fromFile,
+							   CServerPath const& toPath, std::wstring const& toFile)
 	: m_fromPath(fromPath)
 	, m_toPath(toPath)
 	, m_fromFile(fromFile)
@@ -124,7 +128,7 @@ bool CRenameCommand::valid() const
 	return !GetFromPath().empty() && !GetToPath().empty() && !GetFromFile().empty() && !GetToFile().empty();
 }
 
-CChmodCommand::CChmodCommand(const CServerPath& path, const wxString& file, const wxString& permission)
+CChmodCommand::CChmodCommand(CServerPath const& path, std::wstring const& file, std::wstring const& permission)
 	: m_path(path)
 	, m_file(file)
 	, m_permission(permission)
@@ -133,4 +137,4 @@ CChmodCommand::CChmodCommand(const CServerPath& path, const wxString& file, cons
 bool CChmodCommand::valid() const
 {
 	return !GetPath().empty() && !GetFile().empty() && !GetPermission().empty();
-}
\ No newline at end of file
+}
