@@ -13,6 +13,8 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <wx/evtloop.h>
+
 #include "wxfilesystem_blob_handler.h"
 #include "xrc_helper.h"
 #ifdef __WXMSW__
@@ -28,6 +30,10 @@
 
 #include "locale_initializer.h"
 
+#ifdef USE_MAC_SANDBOX
+#include "osx_sandbox_userdirs.h"
+#endif
+
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
 	#include "prefix.h"
@@ -67,14 +73,16 @@ static void SetAppId()
 	if (!dll.Load(_T("shell32.dll")))
 		return;
 
-	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID")))
+	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID"))) {
 		return;
+	}
 
 	t_SetCurrentProcessExplicitAppUserModelID pSetCurrentProcessExplicitAppUserModelID =
 		(t_SetCurrentProcessExplicitAppUserModelID)dll.GetSymbol(_T("SetCurrentProcessExplicitAppUserModelID"));
 
-	if (!pSetCurrentProcessExplicitAppUserModelID)
+	if (!pSetCurrentProcessExplicitAppUserModelID) {
 		return;
+	}
 
 	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
 }
@@ -113,24 +121,28 @@ void CFileZillaApp::InitLocale()
 			wxLocale *loc = wxGetLocale();
 			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
 			if (!loc || !currentInfo) {
-				if (!pInfo)
+				if (!pInfo) {
 					error.Printf(_("Failed to set language to %s, using default system language."),
 						language);
-				else
+				}
+				else {
 					error.Printf(_("Failed to set language to %s (%s), using default system language."),
 						pInfo->Description, language);
+				}
 			}
 			else {
 				wxString currentName = currentInfo->CanonicalName;
 
-				if (!pInfo)
+				if (!pInfo) {
 					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
 						language, loc->GetLocale(),
 						currentName);
-				else
+				}
+				else {
 					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
 						pInfo->Description, language, loc->GetLocale(),
 						currentName);
+				}
 			}
 
 			error += _T("\n");
@@ -141,18 +153,20 @@ void CFileZillaApp::InitLocale()
 		}
 #else
 		if (!pInfo || !SetLocale(pInfo->Language)) {
-			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
+			for (language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language)) {
 				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
-				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
+				if (fallbackInfo && SetLocale(fallbackInfo->Language)) {
 					COptions::Get()->SetOption(OPTION_LANGUAGE, language.ToStdWstring());
 					return;
 				}
 			}
 			COptions::Get()->SetOption(OPTION_LANGUAGE, std::wstring());
-			if (pInfo && !pInfo->Description.empty())
+			if (pInfo && !pInfo->Description.empty()) {
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
-			else
+			}
+			else {
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s, using default system language"), language), _("Failed to change language"), wxICON_EXCLAMATION);
+			}
 		}
 #endif
 	}
@@ -205,8 +219,9 @@ bool CFileZillaApp::OnInit()
 #endif
 
 	int cmdline_result = ProcessCommandLine();
-	if (!cmdline_result)
+	if (!cmdline_result) {
 		return false;
+	}
 
 	LoadLocales();
 
@@ -217,6 +232,16 @@ bool CFileZillaApp::OnInit()
 		return false;
 	}
 
+#if USE_MAC_SANDBOX
+	// Set PUTTYDIR so that fzsftp uses the sandboxed home to put settings.
+	wxString home;
+	if (wxGetEnv(L"HOME", &home) && !home.empty()) {
+		if (home[home.size() - 1] != '/') {
+			home += '/';
+		}
+		wxSetEnv("PUTTYDIR", home + L".config/putty");
+	}
+#endif
 	InitDefaultsDir();
 
 	COptions::Init();
@@ -249,6 +274,9 @@ USE AT OWN RISK"), _T("Important Information"));
 	}
 
 	CheckExistsFzsftp();
+#if ENABLE_STORJ
+	CheckExistsFzstorj();
+#endif
 
 	// Turn off idle events, we don't need them
 	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
@@ -263,12 +291,15 @@ USE AT OWN RISK"), _T("Important Information"));
 	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
+#ifdef USE_MAC_SANDBOX
+	OSXSandboxUserdirs::Get().Load();
+#endif
+
 	CMainFrame *frame = new CMainFrame();
 	frame->Show(true);
 	SetTopWindow(frame);
 
-	CWelcomeDialog *welcome_dialog = new CWelcomeDialog;
-	welcome_dialog->Run(frame, false, true);
+	CWelcomeDialog::RunDelayed(frame);
 
 	frame->ProcessCommandLine();
 	frame->PostInitialize();
@@ -280,11 +311,22 @@ USE AT OWN RISK"), _T("Important Information"));
 
 int CFileZillaApp::OnExit()
 {
+	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUITNOW);
+
 	COptions::Get()->SaveIfNeeded();
 
 #ifdef WITH_LIBDBUS
 	CSessionManager::Uninit();
 #endif
+
+	if (GetMainLoop() && wxEventLoopBase::GetActive() != GetMainLoop()) {
+		// There's open dialogs and such and we just have to quit _NOW_.
+		// We cannot continue as wx proceeds to destroy all windows, which interacts
+		// badly with nested event loops, virtually always causing a crash.
+		// I very much prefer clean shutdown but it's impossible in this situation. Sad.
+		_exit(0);
+	}
+
 #ifdef __WXMSW__
 	UninitWinsock();
 #endif
@@ -431,6 +473,7 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->LoadFile(wxString(dir + L"settings.xrc"));
 	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
 	pResource->LoadFile(wxString(dir + L"update.xrc"));
+	pResource->LoadFile(wxString(dir + L"storj.xrc"));
 
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
@@ -443,7 +486,7 @@ bool CFileZillaApp::InitDefaultsDir()
 	AddStartupProfileRecord("InitDefaultsDir");
 #ifdef __WXGTK__
 	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
-	if( m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
+	if (m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
 		if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml"))) {
 			m_defaultsDir.SetPath(_T("/etc/filezilla"));
 		}
@@ -453,7 +496,7 @@ bool CFileZillaApp::InitDefaultsDir()
 	}
 
 #endif
-	if( m_defaultsDir.empty() ) {
+	if (m_defaultsDir.empty()) {
 		m_defaultsDir = GetDataDir(_T("fzdefaults.xml"));
 	}
 
@@ -465,8 +508,9 @@ bool CFileZillaApp::LoadLocales()
 	AddStartupProfileRecord("CFileZillaApp::LoadLocales");
 #ifndef __WXMAC__
 	m_localesDir = GetDataDir(_T("../locale/de/filezilla.mo"));
-	if (m_localesDir.empty())
+	if (m_localesDir.empty()) {
 		m_localesDir = GetDataDir(_T("../locale/de/LC_MESSAGES/filezilla.mo"));
+	}
 	if (!m_localesDir.empty()) {
 		m_localesDir.ChangePath( _T("../locale") );
 	}
@@ -498,7 +542,7 @@ bool CFileZillaApp::SetLocale(int language)
 	if (!pLocale->IsOk()) {
 		return false;
 	}
-	
+
 	// Load catalog. Only fail if a non-default language has been selected
 	if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
 		return false;
@@ -527,16 +571,18 @@ bool CFileZillaApp::SetLocale(int language)
 
 int CFileZillaApp::GetCurrentLanguage() const
 {
-	if (!m_pLocale)
+	if (!m_pLocale) {
 		return wxLANGUAGE_ENGLISH;
+	}
 
 	return m_pLocale->GetLanguage();
 }
 
 wxString CFileZillaApp::GetCurrentLanguageCode() const
 {
-	if (!m_pLocale)
+	if (!m_pLocale) {
 		return wxString();
+	}
 
 	return m_pLocale->GetCanonicalName();
 }
@@ -550,8 +596,9 @@ void CFileZillaApp::OnFatalException()
 void CFileZillaApp::DisplayEncodingWarning()
 {
 	static bool displayedEncodingWarning = false;
-	if (displayedEncodingWarning)
+	if (displayedEncodingWarning) {
 		return;
+	}
 
 	displayedEncodingWarning = true;
 
@@ -565,36 +612,50 @@ CWrapEngine* CFileZillaApp::GetWrapEngine()
 
 void CFileZillaApp::CheckExistsFzsftp()
 {
-	AddStartupProfileRecord("FileZillaApp::CheckExistsFzstp");
-	// Get the correct path to the fzsftp executable
+	AddStartupProfileRecord("FileZillaApp::CheckExistsFzsftp");
+	CheckExistsTool(L"fzsftp", {L"/src/putty", L"/putty"}, L"FZ_FZSFTP", OPTION_FZSFTP_EXECUTABLE, _("SFTP support"));
+}
+
+#if ENABLE_STORJ
+void CFileZillaApp::CheckExistsFzstorj()
+{
+	AddStartupProfileRecord("FileZillaApp::CheckExistsFzstorj");
+	CheckExistsTool(L"fzstorj", {L"/src/storj", L"/storj"}, L"FZ_FZSTORJ", OPTION_FZSTORJ_EXECUTABLE, _("Storj support"));
+}
+#endif
+
+void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::vector<std::wstring> const& searchPaths, std::wstring const& env, int setting, wxString const& description)
+{
+	// Get the correct path to the specified tool
 
 #ifdef __WXMAC__
 	wxString executable = wxStandardPaths::Get().GetExecutablePath();
 	int pos = executable.Find('/', true);
-	if (pos != -1)
+	if (pos != -1) {
 		executable = executable.Left(pos);
-	executable += _T("/fzsftp");
-	if (!wxFileName::FileExists(executable.ToStdWstring()))
-	{
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable),
+	}
+	executable += _T("/") + tool;
+	if (!wxFileName::FileExists(executable.ToStdWstring())) {
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable, description),
 			_("File not found"), wxICON_ERROR);
 		executable.clear();
 	}
 
 #else
 
-	wxString program = _T("fzsftp");
+	wxString program = tool;
 #ifdef __WXMSW__
 	program += _T(".exe");
 #endif
 
 	bool found = false;
 
-	// First check the FZ_FZSFTP environment variable
+	// First check the given environment variable
 	wxString executable;
-	if (wxGetEnv(_T("FZ_FZSFTP"), &executable)) {
-		if (wxFileName::FileExists(executable.ToStdWstring()))
+	if (wxGetEnv(env, &executable)) {
+		if (wxFileName::FileExists(executable.ToStdWstring())) {
 			found = true;
+		}
 	}
 
 	if (!found) {
@@ -621,12 +682,14 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 		// Add a few paths relative to the current working directory
 		pathList.Add(cwd + _T("/bin"));
-		pathList.Add(cwd + _T("/src/putty"));
-		pathList.Add(cwd + _T("/putty"));
+		for (auto const& path : searchPaths) {
+			pathList.Add(cwd + path);
+		}
 
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty())
+		if (!executable.empty()) {
 			found = true;
+		}
 	}
 
 #ifdef __UNIX__
@@ -652,22 +715,24 @@ void CFileZillaApp::CheckExistsFzsftp()
 		wxPathList pathList;
 		pathList.AddEnvList(_T("PATH"));
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty())
+		if (!executable.empty()) {
 			found = true;
+		}
 	}
 
 	if (!found) {
 		// Quote path if it contains spaces
-		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'') {
 			executable = _T("\"") + executable + _T("\"");
+		}
 
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program, program, program),
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the %s environment variable."), program, description, program, program, env),
 			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
 #endif
 
-	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable.ToStdWstring());
+	COptions::Get()->SetOption(setting, executable.ToStdWstring());
 }
 
 #ifdef __WXMSW__
@@ -698,8 +763,9 @@ int CFileZillaApp::ProcessCommandLine()
 
 		if (m_pCommandLine->HasSwitch(CCommandLine::version)) {
 			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
-			if (!CBuildInfo::GetBuildType().empty())
+			if (!CBuildInfo::GetBuildType().empty()) {
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
+			}
 			out += _T(", compiled on ") + CBuildInfo::GetBuildDateString();
 
 			printf("%s\n", (const char*)out.mb_str());
