@@ -22,6 +22,7 @@
 #include "import.h"
 #include "inputdialog.h"
 #include "led.h"
+#include "list_search_panel.h"
 #include "local_recursive_operation.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
@@ -202,17 +203,19 @@ protected:
 	{
 		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
-			const CServer* pServer = pState ? pState->GetServer() : 0;
-			if (!pServer)
+			if (!pState || !pState->GetServer()) {
 				m_pMainFrame->SetTitle(_T("FileZilla"));
-			else
+			}
+			else {
 				m_pMainFrame->SetTitle(pState->GetTitle() + _T(" - FileZilla"));
+			}
 
 			return;
 		}
 
-		if (!pState)
+		if (!pState) {
 			return;
+		}
 
 		if (!m_pMainFrame->m_pContextControl) {
 			return;
@@ -224,17 +227,18 @@ protected:
 		}
 
 		if (!controls->used()) {
-			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER)
+			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER) {
 				pState->Disconnect();
+			}
 
 			return;
 		}
 
 		if (notification == STATECHANGE_SERVER) {
-			const CServer* pServer = pState->GetServer();
+			ServerWithCredentials const& server = pState->GetServer();
 
 			if (pState == CContextManager::Get()->GetCurrentContext()) {
-				if (!pServer) {
+				if (!server) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
 				}
 				else {
@@ -249,6 +253,49 @@ protected:
 	CMainFrame* m_pMainFrame;
 };
 
+
+/*#include "overlay.h"
+#include <wx/hyperlink.h>
+
+namespace {
+void ShowStorjOverlay(wxWindow* parent, wxWindow* anchor, wxPoint const& offset)
+{
+	auto p = new OverlayWindow(parent);
+
+	auto box = new wxBoxSizer(wxVERTICAL);
+	auto sizer = new wxFlexGridSizer(1, wxSize(5, 5));
+	box->Add(sizer, 0, wxALL, 7);
+
+	p->SetSizer(box);
+
+	auto title = new wxStaticText(p, -1, _("Did you know about this new FileZilla feature?"));
+
+	wxFont f = title->GetFont();
+	f.SetWeight(wxBOLD);
+	title->SetFont(f);
+	sizer->Add(title);
+
+	sizer->AddSpacer(0);
+
+	sizer->Add(new wxStaticText(p, -1, _("You can use FileZilla to store your files securely in the Storj decentralized cloud.")));
+	sizer->Add(new wxHyperlinkCtrl(p, -1, _("Click to learn more"), L"https://wiki.filezilla-project.org/Storj"));
+
+	sizer->AddSpacer(10);
+
+	//sizer->Add(new wxCheckBox(p, -1, L"Don't show this again"), 0, wxALIGN_RIGHT);
+
+	auto dismiss = new wxButton(p, -1, L"Dismiss");
+	dismiss->Bind(wxEVT_BUTTON, [p](wxCommandEvent&) { p->Destroy(); });
+	sizer->Add(dismiss, 0, wxALIGN_RIGHT);
+
+	wxGetApp().GetWrapEngine()->WrapRecursive(p, 3);
+
+	box->Fit(p);
+
+	p->SetAnchor(anchor, offset);
+}
+}*/
+
 CMainFrame::CMainFrame()
 	: m_engineContext(*COptions::Get(), CustomEncodingConverter::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
@@ -490,17 +537,18 @@ void CMainFrame::OnSize(wxSizeEvent &event)
 {
 	wxFrame::OnSize(event);
 
-	if (!m_pBottomSplitter)
+	if (!m_pBottomSplitter) {
 		return;
+	}
 
 	HandleResize();
 
 #ifdef __WXGTK__
-	if (m_pWindowStateManager && m_pWindowStateManager->m_maximize_requested && IsMaximized())
-	{
+	if (m_pWindowStateManager && m_pWindowStateManager->m_maximize_requested && IsMaximized()) {
 		m_pWindowStateManager->m_maximize_requested = 0;
-		if (!RestoreSplitterPositions())
+		if (!RestoreSplitterPositions()) {
 			SetDefaultSplitterPositions();
+		}
 	}
 #endif
 }
@@ -529,16 +577,13 @@ bool CMainFrame::CreateQuickconnectBar()
 	delete m_pQuickconnectBar;
 
 	m_pQuickconnectBar = new CQuickconnectBar();
-	if (!m_pQuickconnectBar->Create(this))
-	{
+	if (!m_pQuickconnectBar->Create(this)) {
 		delete m_pQuickconnectBar;
 		m_pQuickconnectBar = 0;
 	}
-	else
-	{
+	else {
 		wxSize clientSize = GetClientSize();
-		if (m_pTopSplitter)
-		{
+		if (m_pTopSplitter) {
 			wxSize panelSize = m_pQuickconnectBar->GetSize();
 			m_pTopSplitter->SetSize(-1, panelSize.GetHeight(), -1, clientSize.GetHeight() - panelSize.GetHeight(), wxSIZE_USE_EXISTING);
 		}
@@ -557,23 +602,28 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		OpenSiteManager();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_FILE_COPYSITEMANAGER")) {
+		ServerWithCredentials server;
 		CState* pState = CContextManager::Get()->GetCurrentContext();
-		const CServer* pServer = pState ? pState->GetServer() : 0;
-		if (!pServer) {
+		if (pState) {
+			server = pState->GetServer();
+		}
+		if (!server) {
 			wxMessageBoxEx(_("Not connected to any server."), _("Cannot add server to Site Manager"), wxICON_EXCLAMATION);
 			return;
 		}
-		OpenSiteManager(pServer);
+		OpenSiteManager(&server);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_CMD")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
-		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle())
+		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
 			return;
+		}
 
 		CInputDialog dlg;
 		dlg.Create(this, _("Enter custom command"), _("Please enter raw FTP command.\nUsing raw ftp commands will clear the directory cache."));
-		if (dlg.ShowModal() != wxID_OK)
+		if (dlg.ShowModal() != wxID_OK) {
 			return;
+		}
 
 		pState = CContextManager::Get()->GetCurrentContext();
 		if (!pState || !pState->m_pCommandQueue || !pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
@@ -796,11 +846,11 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
+			CNewBookmarkDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
+			CBookmarksDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
@@ -925,7 +975,9 @@ void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 		case nId_listing:
 			{
 				auto const& listingNotification = static_cast<CDirectoryListingNotification const&>(*pNotification.get());
-				pState->m_pCommandQueue->ProcessDirectoryListing(listingNotification);
+				if (pState->m_pCommandQueue) {
+					pState->m_pCommandQueue->ProcessDirectoryListing(listingNotification);
+				}
 			}
 			break;
 		case nId_asyncrequest:
@@ -1088,8 +1140,7 @@ BOOL CALLBACK FzEnumThreadWndProc(HWND hwnd, LPARAM lParam)
 bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 {
 #ifndef __WXMAC__
-	if (m_taskBarIcon)
-	{
+	if (m_taskBarIcon) {
 		delete m_taskBarIcon;
 		m_taskBarIcon = 0;
 		m_closeEvent = event.GetEventType();
@@ -1108,16 +1159,27 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 		wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
 		wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
 		while (pTop != this && (size != prev_size || pLast != pTop)) {
+			if (!pTop) {
+				++iter;
+				if (iter == wxTopLevelWindows.rend()) {
+					break;
+				}
+				pTop = (wxTopLevelWindow*)(*iter);
+				continue;
+			}
+
 			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
-			if (pDialog)
+			if (pDialog) {
 				pDialog->EndModal(wxID_CANCEL);
+			}
 			else {
 				wxWindow* pParent = pTop->GetParent();
 				if (m_pQueuePane && pParent == m_pQueuePane) {
 					// It's the AUI frame manager hint window. Ignore it
 					++iter;
-					if (iter == wxTopLevelWindows.rend())
+					if (iter == wxTopLevelWindows.rend()) {
 						break;
+					}
 					pTop = (wxTopLevelWindow*)(*iter);
 					continue;
 				}
@@ -1139,8 +1201,7 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 	// wxMessageBoxEx does not use wxTopLevelWindow, close it too
 	bool dialog = false;
 	EnumThreadWindows(GetCurrentThreadId(), FzEnumThreadWndProc, (LPARAM)&dialog);
-	if (dialog)
-	{
+	if (dialog) {
 		m_closeEvent = event.GetEventType();
 		m_closeEventTimer.Start(1, true);
 
@@ -1262,8 +1323,9 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		}
 
 		if (pState->m_pCommandQueue) {
-			if (!pState->m_pCommandQueue->Quit())
+			if (!pState->m_pCommandQueue->Quit()) {
 				res = false;
+			}
 		}
 	}
 
@@ -1277,7 +1339,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 	if (controls) {
 		Site site = controls->pState->GetLastSite();
-		COptions::Get()->SetLastServer(site.m_server);
+		COptions::Get()->SetLastServer(site.server_);
 		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, controls->pState->GetLastServerPath().GetSafePath());
 		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, site.m_path);
 	}
@@ -1347,7 +1409,7 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(CServer const* pServer)
+void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState) {
@@ -1382,7 +1444,7 @@ void CMainFrame::OpenSiteManager(CServer const* pServer)
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = site.m_server;
+		connected_site.server = site.server_;
 		connected_sites.push_back(connected_site);
 		handled_paths.insert(path);
 	}
@@ -1451,8 +1513,9 @@ void CMainFrame::OnProcessQueue(wxCommandEvent& event)
 void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 {
 	CSettingsDialog dlg(m_engineContext);
-	if (!dlg.Create(this))
+	if (!dlg.Create(this)) {
 		return;
+	}
 
 	COptions* pOptions = COptions::Get();
 
@@ -1481,8 +1544,9 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 
 void CMainFrame::OnToggleLogView(wxCommandEvent&)
 {
-	if (!m_pTopSplitter)
+	if (!m_pTopSplitter) {
 		return;
+	}
 
 	bool shown;
 
@@ -1924,12 +1988,8 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark)
 {
 	// First check if we need to ask user for a password
-	if (data.m_server.GetLogonType() == ASK ||
-		(data.m_server.GetLogonType() == INTERACTIVE && data.m_server.GetUser().empty()))
-	{
-		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName())) {
-			return false;
-		}
+	if (!CLoginManager::Get().GetPassword(data.server_, false, data.server_.server.GetName())) {
+		return false;
 	}
 
 	// Check if current state is already connected and if needed ask whether to open in new tab
@@ -2241,6 +2301,21 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 			return;
 		}
 
+		if (controls->pLocalListSearchPanel->IsShown() || controls->pRemoteListSearchPanel->IsShown()) {
+			CConditionalDialog dlg(this, CConditionalDialog::quick_search, CConditionalDialog::yesno);
+			dlg.SetTitle(_("Directory comparison"));
+			dlg.AddText(_("To compare directories quick search must be closed."));
+			dlg.AddText(_("Close quick search and continue comparing?"));
+			if (!dlg.Run())
+				return;
+
+			if (controls->pLocalListSearchPanel->IsShown())
+				controls->pLocalListSearchPanel->Close();
+
+			if (controls->pRemoteListSearchPanel->IsShown())
+				controls->pRemoteListSearchPanel->Close();
+		}
+
 		if ((controls->pLocalSplitter->IsSplit() && !controls->pRemoteSplitter->IsSplit()) ||
 			(!controls->pLocalSplitter->IsSplit() && controls->pRemoteSplitter->IsSplit()))
 		{
@@ -2394,14 +2469,14 @@ void CMainFrame::ProcessCommandLine()
 	if (!param.empty()) {
 		std::wstring error;
 
-		CServer server;
+		ServerWithCredentials server;
 
 		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
 		if (logontype == _T("ask")) {
-			server.SetLogonType(ASK);
+			server.SetLogonType(LogonType::ask);
 		}
 		else if (logontype == _T("interactive")) {
-			server.SetLogonType(INTERACTIVE);
+			server.SetLogonType(LogonType::interactive);
 		}
 
 		CServerPath path;
@@ -2411,20 +2486,16 @@ void CMainFrame::ProcessCommandLine()
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.GetLogonType() == NORMAL) {
-			server.SetLogonType(ASK);
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
+			server.SetLogonType(LogonType::ask);
 			CLoginManager::Get().RememberPassword(server);
 		}
-		else if (server.GetLogonType() == ASK ||
-			(server.GetLogonType() == INTERACTIVE && server.GetUser().empty()))
-		{
-			if (!CLoginManager::Get().GetPassword(server, false)) {
-				return;
-			}
+		else if (!CLoginManager::Get().GetPassword(server, false)) {
+			return;
 		}
 
 		Site site;
-		site.m_server = server;
+		site.server_ = server;
 		Bookmark bm;
 		bm.m_remoteDir = path;
 		ConnectToSite(site, bm);
@@ -2596,15 +2667,18 @@ void CMainFrame::PostInitialize()
 
 #if FZ_MANUALUPDATECHECK
 	// Need to do this after welcome screen to avoid simultaneous display of multiple dialogs
-	if( !m_pUpdater ) {
+	if (!m_pUpdater) {
 		update_dialog_timer_.SetOwner(this);
-		m_pUpdater = new CUpdater(*this, m_engineContext);
+		m_pUpdater = new CUpdater(*this, m_engineContext,
+			[this](CActiveNotification const& notification) {
+				UpdateActivityLed(notification.GetDirection());
+			}
+		);
 		m_pUpdater->Init();
 	}
 #endif
 
-	if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0)
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) != 0) {
 		Show();
 		OpenSiteManager();
 	}
@@ -2612,14 +2686,16 @@ void CMainFrame::PostInitialize()
 
 void CMainFrame::OnMenuNewTab(wxCommandEvent&)
 {
-	if (m_pContextControl)
+	if (m_pContextControl) {
 		m_pContextControl->CreateTab();
+	}
 }
 
 void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 {
-	if (!m_pContextControl)
+	if (!m_pContextControl) {
 		return;
+	}
 
 	m_pContextControl->CloseTab(m_pContextControl->GetCurrentTab());
 }
