@@ -198,6 +198,9 @@ EVT_MENU(XRCID("ID_PRIORITY_LOWEST"), CQueueView::OnSetPriority)
 EVT_COMMAND(wxID_ANY, fzEVT_GRANTEXCLUSIVEENGINEACCESS, CQueueView::OnExclusiveEngineRequestGranted)
 
 EVT_SIZE(CQueueView::OnSize)
+
+EVT_LIST_COL_CLICK(wxID_ANY, CQueueView::OnColumnClicked)
+
 END_EVENT_TABLE()
 
 CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
@@ -205,6 +208,8 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	m_pMainFrame(pMainFrame),
 	m_pAsyncRequestQueue(pAsyncRequestQueue)
 {
+	wxGetApp().AddStartupProfileRecord("CQueueView::CQueueView");
+
 	if (m_pAsyncRequestQueue) {
 		m_pAsyncRequestQueue->SetQueue(this);
 	}
@@ -218,8 +223,7 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 	}
 	m_actionAfterState = static_cast<ActionAfterState::type>(action);
 
-	std::list<ColumnId> extraCols;
-	extraCols.push_back(colTransferStatus);
+	std::vector<ColumnId> extraCols({colTransferStatus});
 	CreateColumns(extraCols);
 
 	RegisterOption(OPTION_NUMTRANSFERS);
@@ -319,7 +323,7 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 {
 	CServerItem* pServerItem = CreateServerItem(dataObject.GetServer());
 
-	const std::list<CRemoteDataObject::t_fileInfo>& files = dataObject.GetFiles();
+	std::vector<CRemoteDataObject::t_fileInfo> const& files = dataObject.GetFiles();
 
 	for (auto const& fileInfo : files) {
 		if (fileInfo.dir) {
@@ -1590,6 +1594,7 @@ void CQueueView::LoadQueueFromXML()
 
 void CQueueView::LoadQueue()
 {
+	wxGetApp().AddStartupProfileRecord("CQueueView::LoadQueue");
 	// We have to synchronize access to queue.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
@@ -3179,3 +3184,16 @@ void CQueueView::OnStateChange(CState*, t_statechange_notifications notification
 		}
 	}
 }
+
+void CQueueView::OnColumnClicked(wxListEvent &event)
+{
+	int const col = event.GetColumn();
+	bool const reverse = wxGetKeyState(WXK_SHIFT);
+
+	for (auto * serverItem : m_serverList) {
+		serverItem->Sort(col, reverse);
+	}
+
+	RefreshListOnly();
+	UpdateStatusLinePositions();
+}
