@@ -1,6 +1,7 @@
 #include <filezilla.h>
 #include "StatusView.h"
 #include "Options.h"
+#include "state.h"
 
 #include <wx/dcclient.h>
 
@@ -35,7 +36,7 @@ public:
 		::SendMessage((HWND)GetHandle(), EM_REPLACESEL, 0, (LPARAM)_T(""));
 	}
 
-	void AppendText(const wxString& text, int lineCount, const CHARFORMAT2& cf)
+	void AppendText(std::wstring const& text, int lineCount, const CHARFORMAT2& cf)
 	{
 		HWND hwnd = (HWND)GetHWND();
 
@@ -45,7 +46,7 @@ public:
 		::SendMessage(hwnd, EM_EXSETSEL, 0, (LPARAM)&range);
 		::SendMessage(hwnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
 		m_updatesCount = -2; // suppress any update event
-		::SendMessage(hwnd, EM_REPLACESEL, 0, reinterpret_cast<LPARAM>(static_cast<wxChar const*>(text.c_str())));
+		::SendMessage(hwnd, EM_REPLACESEL, 0, reinterpret_cast<LPARAM>(text.c_str()));
 		::SendMessage(hwnd, EM_LINESCROLL, (WPARAM)0, (LPARAM)lineCount);
 	}
 #endif
@@ -129,7 +130,7 @@ void CStatusView::AddToLog(CLogmsgNotification const& notification)
 	AddToLog(notification.msgType, notification.msg, fz::datetime::now());
 }
 
-void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz::datetime const& time)
+void CStatusView::AddToLog(MessageType messagetype, std::wstring const& message, fz::datetime const& time)
 {
 	if (!m_shown) {
 		if (m_hiddenLines.size() >= MAX_LINECOUNT) {
@@ -149,16 +150,16 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz:
 		return;
 	}
 
-	const int messageLength = message.Length();
+	size_t const messageLength = message.size();
 
-	wxString prefix;
-	prefix.Alloc(25 + messageLength);
+	// This does not clear storage
+	m_formattedMessage.clear();
 
 	if (m_nLineCount)
 #ifdef __WXMSW__
-		prefix = _T("\r\n");
+		m_formattedMessage = _T("\r\n");
 #else
-		prefix = _T("\n");
+		m_formattedMessage = _T("\n");
 #endif
 
 	if (m_nLineCount >= MAX_LINECOUNT) {
@@ -179,7 +180,7 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz:
 	}
 #endif
 
-	int lineLength = m_attributeCache[static_cast<int>(messagetype)].len + messageLength;
+	size_t lineLength = m_attributeCache[static_cast<int>(messagetype)].len + messageLength;
 
 	if (m_showTimestamps) {
 		if (time != m_lastTime) {
@@ -191,8 +192,8 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz:
 			m_lastTimeString = time.format(_T("%H:%M:%S "), fz::datetime::local);
 #endif
 		}
-		prefix += m_lastTimeString;
-		lineLength += m_lastTimeString.Len();
+		m_formattedMessage += m_lastTimeString;
+		lineLength += m_lastTimeString.size();
 	}
 
 #ifdef __WXMAC__
@@ -201,7 +202,7 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz:
 	m_pTextCtrl->SetDefaultColor(m_attributeCache[static_cast<int>(messagetype)].attr.GetTextColour());
 #endif
 
-	prefix += m_attributeCache[static_cast<int>(messagetype)].prefix;
+	m_formattedMessage += m_attributeCache[static_cast<int>(messagetype)].prefix;
 
 	if (m_rtl) {
 		// Unicode control characters that control reading direction
@@ -216,24 +217,28 @@ void CStatusView::AddToLog(MessageType messagetype, const wxString& message, fz:
 		if (messagetype == MessageType::Command || messagetype == MessageType::Response || messagetype >= MessageType::Debug_Warning) {
 			// Commands, responses and debug message contain English text,
 			// set LTR reading order for them.
-			prefix += LTR_MARK;
-			prefix += LTR_EMBED;
+			m_formattedMessage += LTR_MARK;
+			m_formattedMessage += LTR_EMBED;
 			lineLength += 2;
 		}
 	}
 
-	prefix += message;
+	m_formattedMessage += message;
 #if defined(__WXGTK__)
 	// AppendText always calls SetInsertionPointEnd, which is very expensive.
 	// This check however is negligible.
 	if (m_pTextCtrl->GetInsertionPoint() != m_pTextCtrl->GetLastPosition())
-		m_pTextCtrl->AppendText(prefix);
+		m_pTextCtrl->AppendText(m_formattedMessage);
 	else
-		m_pTextCtrl->WriteText(prefix);
+		m_pTextCtrl->WriteText(m_formattedMessage);
+	#ifdef __WXGTK3__
+		// Some smooth scrolling oddities prevent auto-scrolling. Manuall tell it to scroll.
+		m_pTextCtrl->ShowPosition(m_pTextCtrl->GetInsertionPoint());
+	#endif
 #elif defined(__WXMAC__)
-	m_pTextCtrl->WriteText(prefix);
+	m_pTextCtrl->WriteText(m_formattedMessage);
 #else
-	m_pTextCtrl->AppendText(prefix, m_nLineCount, m_attributeCache[static_cast<int>(messagetype)].cf);
+	m_pTextCtrl->AppendText(m_formattedMessage, m_nLineCount, m_attributeCache[static_cast<int>(messagetype)].cf);
 #endif
 
 	if (m_nLineCount >= MAX_LINECOUNT) {
@@ -339,18 +344,18 @@ void CStatusView::InitDefAttr()
 #endif
 		switch (static_cast<MessageType>(i)) {
 		case MessageType::Error:
-			entry.prefix = _("Error:");
+			entry.prefix = _("Error:").ToStdWstring();
 			entry.attr.SetTextColour(wxColour(255, 0, 0));
 			break;
 		case MessageType::Command:
-			entry.prefix = _("Command:");
+			entry.prefix = _("Command:").ToStdWstring();
 			if (is_dark)
 				entry.attr.SetTextColour(wxColour(128, 128, 255));
 			else
 				entry.attr.SetTextColour(wxColour(0, 0, 128));
 			break;
 		case MessageType::Response:
-			entry.prefix = _("Response:");
+			entry.prefix = _("Response:").ToStdWstring();
 			if (is_dark)
 				entry.attr.SetTextColour(wxColour(128, 255, 128));
 			else
@@ -360,22 +365,22 @@ void CStatusView::InitDefAttr()
 		case MessageType::Debug_Info:
 		case MessageType::Debug_Verbose:
 		case MessageType::Debug_Debug:
-			entry.prefix = _("Trace:");
+			entry.prefix = _("Trace:").ToStdWstring();
 			if (is_dark)
 				entry.attr.SetTextColour(wxColour(255, 128, 255));
 			else
 				entry.attr.SetTextColour(wxColour(128, 0, 128));
 			break;
 		case MessageType::RawList:
-			entry.prefix = _("Listing:");
+			entry.prefix = _("Listing:").ToStdWstring();
 			if (is_dark)
 				entry.attr.SetTextColour(wxColour(128, 255, 255));
 			else
 				entry.attr.SetTextColour(wxColour(0, 128, 128));
 			break;
 		default:
-			entry.prefix = _("Status:");
-			entry.attr.SetTextColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
+			entry.prefix = _("Status:").ToStdWstring();
+			entry.attr.SetTextColour(wxSystemSettings::GetColour(wxSYS_COLOUR_LISTBOXTEXT));
 			break;
 		}
 
@@ -384,10 +389,10 @@ void CStatusView::InitDefAttr()
 		dc.GetTextExtent(entry.prefix, &width, &height);
 		wxASSERT(width <= maxPrefixWidth);
 		wxCoord spaces = (maxPrefixWidth - width) / spaceWidth;
-		entry.prefix += wxString(spaces, ' ');
+		entry.prefix += std::wstring(spaces, ' ');
 #endif
 		entry.prefix += _T("\t");
-		entry.len = entry.prefix.Length();
+		entry.len = entry.prefix.size();
 
 #ifdef __WXMSW__
 		m_pTextCtrl->SetStyle(m_pTextCtrl->GetInsertionPoint(), m_pTextCtrl->GetInsertionPoint(), entry.attr);
@@ -414,6 +419,19 @@ void CStatusView::OnContextMenu(wxContextMenuEvent&)
 
 	pMenu->Check(XRCID("ID_SHOW_DETAILED_LOG"), COptions::Get()->GetOptionVal(OPTION_LOGGING_SHOW_DETAILED_LOGS) != 0);
 
+	ServerWithCredentials server;
+	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (pState) {
+		auto pItem = pMenu->FindItem(XRCID("ID_MENU_SERVER_CMD"));
+		server = pState->GetServer();
+		if (!server || CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::EnterCommand)) {
+			pItem->Enable(true);
+		}
+		else {
+			pItem->Enable(false);
+		}
+	}
+
 	PopupMenu(pMenu);
 
 	COptions::Get()->SetOption(OPTION_LOGGING_SHOW_DETAILED_LOGS, pMenu->IsChecked(XRCID("ID_SHOW_DETAILED_LOG")) ? 1 : 0);
