@@ -1,5 +1,5 @@
-#ifndef __THEMEPROVIDER_H__
-#define __THEMEPROVIDER_H__
+#ifndef FILEZILLA_INTERFACE_THEMEPROVIDER_HEADER
+#define FILEZILLA_INTERFACE_THEMEPROVIDER_HEADER
 
 #include <option_change_event_handler.h>
 #include <memory>
@@ -8,6 +8,7 @@
 
 enum iconSize
 {
+	iconSizeTiny,
 	iconSizeSmall,
 	iconSize24,
 	iconSizeNormal,
@@ -66,7 +67,7 @@ private:
 	std::map<std::wstring, cacheEntry> cache_;
 };
 
-class CThemeProvider : public wxArtProvider, protected COptionChangeEventHandler
+class CThemeProvider final : public wxArtProvider, protected COptionChangeEventHandler
 {
 public:
 	CThemeProvider();
@@ -79,6 +80,9 @@ public:
 
 	static wxSize GetIconSize(iconSize size, bool userScaled = false);
 
+	// Note: Always 1 on OS X
+	static double GetUIScaleFactor();
+
 	static CThemeProvider* Get();
 
 	wxAnimation CreateAnimation(wxArtID const& id, wxSize const& size);
@@ -92,4 +96,4 @@ protected:
 	std::map<std::wstring, CTheme> themes_;
 };
 
-#endif //__THEMEPROVIDER_H__
+#endif
