@@ -13,6 +13,8 @@
 #include <wx/combobox.h>
 #include <wx/dcclient.h>
 
+#include <algorithm>
+
 #ifdef __WXMSW__
 const int border_offset = 0;
 #elif defined(__WXMAC__)
@@ -464,15 +466,17 @@ void CLocalViewHeader::OnSelectionChanged(wxCommandEvent& event)
 	m_autoCompletionText = _T("");
 #endif
 
-	wxString dir = event.GetString();
-	if (dir.empty())
+	std::wstring dir = event.GetString().ToStdWstring();
+	if (dir.empty()) {
 		return;
+	}
 
 	if (!wxDir::Exists(dir)) {
 		const wxString& current = m_state.GetLocalDir().GetPath();
 		int item = m_pComboBox->FindString(current, true);
-		if (item != wxNOT_FOUND)
+		if (item != wxNOT_FOUND) {
 			m_pComboBox->SetSelection(item);
+		}
 
 		wxBell();
 		return;
@@ -487,14 +491,16 @@ void CLocalViewHeader::OnTextEnter(wxCommandEvent&)
 	m_autoCompletionText = _T("");
 #endif
 
-	wxString dir = m_pComboBox->GetValue();
+	std::wstring dir = m_pComboBox->GetValue().ToStdWstring();
 
-	wxString error;
+	std::wstring error;
 	if (!m_state.SetLocalDir(dir, &error)) {
-		if (!error.empty())
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
+		}
 		m_pComboBox->SetValue(m_state.GetLocalDir().GetPath());
 	}
 }
@@ -539,12 +545,12 @@ void CRemoteViewHeader::OnStateChange(t_statechange_notifications notification,
 			Disable();
 		}
 		else {
-			const CServer* const pServer = m_state.GetServer();
-			if (pServer && *pServer != m_lastServer) {
+			ServerWithCredentials const& server = m_state.GetServer();
+			if (server && server.server != m_lastServer) {
 				m_pComboBox->Clear();
 				m_recentDirectories.clear();
 				m_sortedRecentDirectories.clear();
-				m_lastServer = *pServer;
+				m_lastServer = server.server;
 			}
 			Enable();
 #ifdef __WXGTK__
@@ -559,7 +565,7 @@ void CRemoteViewHeader::OnTextEnter(wxCommandEvent&)
 {
 	CServerPath path = m_path;
 	wxString value = m_pComboBox->GetValue();
-	if (value.empty() || !path.ChangePath(value)) {
+	if (value.empty() || !path.ChangePath(value.ToStdWstring())) {
 		wxBell();
 		return;
 	}
@@ -579,7 +585,7 @@ void CRemoteViewHeader::OnSelectionChanged(wxCommandEvent& event)
 		return;
 
 	CServerPath path = m_path;
-	if (!path.SetPath(dir)) {
+	if (!path.SetPath(dir.ToStdWstring())) {
 		wxBell();
 		return;
 	}
